from kakeibox_core.use_cases.transaction_subcategory.list_all import \
    ListAllTransactionSubcategories
from kakeibox_storage_memory.storage import StorageTransactionSubcategory
from kakeibox_storage_memory.bridge import Bridge


def before_execute_test():
    storage = StorageTransactionSubcategory()
    storage.table = {}


def test_use_case_transaction_subcategory_list(transaction_subcategory_data):
    before_execute_test()
    storage = StorageTransactionSubcategory()
    storage.table = transaction_subcategory_data
    transaction_categories = list(storage.table.values())

    storage_bridge = Bridge()
    use_case = ListAllTransactionSubcategories(storage_bridge)
    result = use_case.execute()
    assert result
    assert isinstance(result.value, list)
    assert transaction_categories == result.value
    for element in result.value:
        assert isinstance(element, dict)



from kakeibox_core.use_cases.transaction_subcategory.list_per_category import \
    ListTransactionSubcategoriesPerCategory
from kakeibox_storage_memory.storage import StorageTransactionSubcategory
from kakeibox_storage_memory.bridge import Bridge


def before_execute_test():
    storage = StorageTransactionSubcategory()
    storage.table = {}


def _get_category_subcategories(storage, category):
    return [item for item in storage.table.values()
            if item['transaction_category_code'] == category
            ]


def _verify_category(storage, category):
    transaction_subcategories = _get_category_subcategories(storage, category)
    first_subcategory = transaction_subcategories[0]
    storage_bridge = Bridge()
    use_case = ListTransactionSubcategoriesPerCategory(storage_bridge)
    result = use_case.execute(category)
    assert result
    assert isinstance(result.value, list)
    assert transaction_subcategories == result.value
    assert first_subcategory['transaction_category_code'] == category
    for element in result.value:
        assert isinstance(element, dict)


def test_use_case_transaction_subcategory_list_per_category(
        transaction_subcategory_data):
    before_execute_test()
    storage = StorageTransactionSubcategory()
    storage.table = transaction_subcategory_data

    for transaction_subcategory in transaction_subcategory_data.values():
        category = transaction_subcategory['transaction_category_code']
        _verify_category(storage, category)



from kakeibox_core.use_cases.transaction_category.new import \
    NewTransactionCategory
from kakeibox_storage_memory.storage import StorageTransactionCategory
from kakeibox_storage_memory.bridge import Bridge


def before_execute_test():
    storage = StorageTransactionCategory()
    storage.table = {}


def test_use_case_transaction_category_new(transaction_category_data):
    before_execute_test()
    transaction_category = list(transaction_category_data.values())[0]
    storage = StorageTransactionCategory()
    assert 0 == len(storage.table)

    storage_bridge = Bridge()
    use_case = NewTransactionCategory(storage_bridge)
    result = use_case.execute(transaction_category)
    assert result
    assert isinstance(result.value, dict)
    assert 1 == len(storage.table)

    new_one = storage.table[result.value['code']]
    assert result.value['code'] == new_one['code']
    assert result.value['name'] == new_one['name']



from kakeibox_core.use_cases.transaction import *
from kakeibox_core.use_cases.transaction_category import *
from kakeibox_core.use_cases.transaction_subcategory import *
from kakeibox_core.use_cases.income import *
from kakeibox_core.use_cases.expense import *
from kakeibox_core.use_cases.saving import *



from kakeibox_core.use_cases.transaction.list import ListTransactions
from kakeibox_storage_memory.storage import StorageTransaction
from kakeibox_storage_memory.bridge import Bridge
import datetime


def before_execute_test():
    storage = StorageTransaction()
    storage.table = {}


def test_use_case_transaction_list(transaction_data):
    before_execute_test()
    end_time = datetime.datetime.now()
    start_time = (end_time - datetime.timedelta(days=10)).timestamp()
    end_time = end_time.timestamp()

    storage = StorageTransaction()
    storage.table = transaction_data
    expenses = [item for item in storage.table.values() if int(start_time) <=
                int(item['timestamp']) <= int(end_time)
                ]
    storage_bridge = Bridge()
    use_case = ListTransactions(storage_bridge)
    result = use_case.execute(start_time, end_time)
    assert result
    assert isinstance(result.value, list)
    assert expenses == result.value
    for element in result.value:
        assert isinstance(element, dict)



import pytest
import datetime
from faker import Faker
from tests.fixtures.kakeibox_core_transaction_data import TRANSACTION_DATA


@pytest.fixture()
def transaction_category_data():
    return {
        'SUR':
            {
                'code': 'SUR',
                'name': 'Survival'
            },
        'OPT':
            {
                'code': 'OPT',
                'name': 'Optional'
            },
        'CUL':
            {
                'code': 'CUL',
                'name': 'Culture'
            },
        'EXT':
            {
                'code': 'EXT',
                'name': 'Extra'
            }
    }


@pytest.fixture()
def transaction_subcategory_data():
    return {
        "FOD": {
            "code": "FOD",
            "name": "Food",
            "transaction_category_code": 'SUR',
        },
        "REN": {
            "code": "REN",
            "name": "Rent",
            "transaction_category_code": 'SUR',
        },
        "TRA": {
            "code": "TRA",
            "name": "Transport",
            "transaction_category_code": 'SUR',
        },
        "KID": {
            "code": "KID",
            "name": "Kids",
            "transaction_category_code": 'SUR',
        },
        "RES": {
            "code": "RES",
            "name": "Restaurant",
            "transaction_category_code": 'OPT',
        },
        "SHO": {
            "code": "SHO",
            "name": "Shopping",
            "transaction_category_code": 'OPT',
        },
        "BOK": {
            "code": "BOK",
            "name": "Books",
            "transaction_category_code": 'CUL',
        },
        "MUS": {
            "code": "MUS",
            "name": "Music",
            "transaction_category_code": 'CUL',
        },
        "SHW": {
            "code": "SHW",
            "name": "Shows",
            "transaction_category_code": 'CUL',
        },
        "MOV": {
            "code": "MOV",
            "name": "Movies",
            "transaction_category_code": 'CUL',
        },
        "MAG": {
            "code": "MAG",
            "name": "Magazines",
            "transaction_category_code": 'CUL',
        },
        "GIF": {
            "code": "GIF",
            "name": " Gifts",
            "transaction_category_code": 'EXT',
        },
        "REP": {
            "code": "REP",
            "name": "Repairs",
            "transaction_category_code": 'EXT',
        },
        "FUR": {
            "code": "FUR",
            "name": "Furniture",
            "transaction_category_code": 'EXT',
        }
    }


def _create_fake_transaction(transaction_type_code=None, amount=None):
    fake = Faker()
    if not transaction_type_code:
        transaction_type_code='EXP'
    end_datetime = datetime.datetime.now()
    start_datetime = end_datetime - datetime.timedelta(days=10)
    if not amount:
        amount = fake.pydecimal(positive=True, min_value=0, max_value=100000)

    return dict(
        transaction_type_code=transaction_type_code,
        transaction_subcategory_code='FOD',
        uuid=fake.uuid4(),
        description=fake.sentence(),
        amount=amount,
        timestamp=fake.unix_time(start_datetime=start_datetime,
                                 end_datetime=end_datetime),
        reference_number=fake.bban(),
        record_hash=''
    )


def _create_fake_expense(amount=None):
    return _create_fake_transaction(amount=amount)


def _create_fake_income(amount=None):
    transaction_type_code = 'INC'
    return _create_fake_transaction(
        transaction_type_code=transaction_type_code, amount=amount)


@pytest.fixture()
def transaction_data():
    return TRANSACTION_DATA


@pytest.fixture()
def an_expense_data():
    return _create_fake_expense()


@pytest.fixture()
def an_income_data():
    return _create_fake_income()


@pytest.fixture()
def transaction_total_data():
    transactions = {}
    for _ in range(1, 11):
        item = _create_fake_income(1000.10)
        transactions.setdefault(item['uuid'], item)

    for _ in range(1, 11):
        item = _create_fake_expense(100.20)
        transactions.setdefault(item['uuid'], item)
    return transactions



from kakeibox_core.use_cases.expense.get import GetExpenseByUUID
from kakeibox_storage_memory.bridge import Bridge
from kakeibox_storage_memory.storage import StorageTransaction


def before_execute_test():
    storage = StorageTransaction()
    storage.table = {}


def test_use_case_expense_get(an_expense_data):
    before_execute_test()
    item = an_expense_data
    storage = StorageTransaction()
    storage.table[item['uuid']] = item
    assert 1 == len(storage.table)

    storage_bridge = Bridge()
    use_case = GetExpenseByUUID(storage_bridge)
    result = use_case.execute(item['uuid'])
    expected = storage.table[item['uuid']]

    assert result
    assert isinstance(result.value, dict)
    assert expected['uuid'] == result.value['uuid']
    assert expected['description'] == result.value['description']
    assert expected['amount'] == result.value['amount']
    assert expected['timestamp'] == result.value['timestamp']
    assert expected['reference_number'] == result.value['reference_number']
    assert expected['record_hash'] == result.value['record_hash']
    assert expected['transaction_type_code'] == result.value[
        'transaction_type_code']
    assert expected['transaction_subcategory_code'] == result.value[
        'transaction_subcategory_code']



class UseCaseContext(object):

    def __init__(self, strategy):
        self._strategy = strategy

    @property
    def strategy(self):
        return self._strategy

    @strategy.setter
    def strategy(self, strategy):
        self._strategy = strategy

    def execute(self) -> None:
        return self._strategy.do()


class UseCaseStrategy(object):

    def __init__(self, storage_bridge):
        self.storage_data_bridge = storage_bridge

    def execute(self):
        raise Exception("Not implemented yet")



from dependency_injector import containers, providers
from kakeibox_storage_sqlite3.bridge import Bridge
from kakeibox_storage_memory.bridge import Bridge as TestBridge


class Adapters(containers.DeclarativeContainer):
    storage_bridge = providers.Factory(Bridge)


class TestAdapters(containers.DeclarativeContainer):
    storage_bridge = providers.Factory(TestBridge)



from kakeibox_core.use_cases.transaction_category.get import \
    TransactionCategoryByCode
from kakeibox_storage_memory.storage import StorageTransactionCategory
from kakeibox_storage_memory.bridge import Bridge


def before_execute_test():
    storage = StorageTransactionCategory()
    storage.table = {}


def test_use_case_transaction_category_get(transaction_category_data):
    before_execute_test()
    item = list(transaction_category_data.values())[0]
    storage = StorageTransactionCategory()
    storage.table[item['code']] = item
    assert 1 == len(storage.table)

    storage_bridge = Bridge()
    use_case = TransactionCategoryByCode(storage_bridge)
    result = use_case.execute(item['code'])
    assert isinstance(result.value, dict)
    assert item['code'] == result.value['code']
    assert item['name'] == result.value['name']



# -*- coding: utf-8 -*-

"""Top-level package for kakeibox."""

__all__ = ['entry_points', 'plugins', 'responses', 'storage_data_bridge',
           'use_cases']
__author__ = """Jonathan López"""
__email__ = 'jlopez@fipasoft.com.mx'
__version__ = '0.9.1'



from kakeibox_core.use_cases.transaction_category.list import \
    ListTransactionCategories
from kakeibox_storage_memory.storage import StorageTransactionCategory
from kakeibox_storage_memory.bridge import Bridge


def before_execute_test():
    storage = StorageTransactionCategory()
    storage.table = {}


def test_use_case_transaction_category_list(transaction_category_data):
    before_execute_test()
    storage = StorageTransactionCategory()
    storage.table = transaction_category_data
    transaction_categories = list(storage.table.values())
    storage_brdige = Bridge()
    use_case = ListTransactionCategories(storage_brdige)
    result = use_case.execute()
    assert transaction_categories == result.value
    assert result
    assert isinstance(result.value, list)
    for element in result.value:
        assert isinstance(element, dict)



from kakeibox_core.use_cases.transaction_subcategory.delete import \
    DeleteTransactionSubcategory
from kakeibox_storage_memory.storage import StorageTransactionSubcategory
from kakeibox_storage_memory.bridge import Bridge


def before_execute_test():
    storage = StorageTransactionSubcategory()
    storage.table = {}


def test_use_case_transaction_subcategory_delete(transaction_subcategory_data):
    before_execute_test()
    item = list(transaction_subcategory_data.values())[0]
    storage = StorageTransactionSubcategory()
    storage.table[item['code']] = item
    assert 1 == len(storage.table)

    storage_bridge = Bridge()
    use_case = DeleteTransactionSubcategory(storage_bridge)
    result = use_case.execute(item['code'])
    assert isinstance(result.value, bool)
    assert result.value
    assert 0 == len(storage.table)



TRANSACTION_DATA = {
    "65725a55-0610-4aa2-b4d1-dee8a0936ed5": {
        "amount": 41778.94451,
        "description": "Focus executive drop cost base my prove night.",
        "record_hash":
            "0b8ea21f957d7fc452ab3d4cd4cb38be394d17f0fb17d0ef62b28b83e6fe7e37",
        "reference_number": "XGTM5260521978861",
        "timestamp": 1563533343,
        "transaction_subcategory_code": 'GIF',
        "transaction_type_code": 'INC',
        "uuid": "65725a55-0610-4aa2-b4d1-dee8a0936ed5"
    },
    "e81bdd0d-b9f8-4721-9708-8a5a3569d6e0": {
        "amount": 6220.522,
        "description":
            "Pick together investment prove police parent election.",
        "record_hash":
            "5c0b7a22276db776c043ca715c659d41030fab651d9f7cace4da04d280e1693e",
        "reference_number": "VEJT4503634011670",
        "timestamp": 1564510129,
        "transaction_subcategory_code": 'BOK',
        "transaction_type_code": 'INC',
        "uuid": "e81bdd0d-b9f8-4721-9708-8a5a3569d6e0"
    },
    "e209ebd3-4748-42fc-a282-8964b0795d45": {
        "amount": 59563.5,
        "description": "Most after table sort anyone.",
        "record_hash":
            "76a20aaa18d4c06a5d3bc328d1cbf658d54e73b1ce5752a6569a76b4508d9439",
        "reference_number": "LZEF2713341746274",
        "timestamp": 1565855698,
        "transaction_subcategory_code": 'SHW',
        "transaction_type_code": 'EXP',
        "uuid": "e209ebd3-4748-42fc-a282-8964b0795d45"
    },
    "06f7eb1c-f1c5-42a9-94df-eaf384a91753": {
        "amount": 76979.1673,
        "description": "Through star safe teacher animal.",
        "record_hash":
            "6f15d625f40d0ed05f3b684c03dbcc2c5e3739c5fcba16989e6f54365d141c3d",
        "reference_number": "MMHL2082573530388",
        "timestamp": 1563080480,
        "transaction_subcategory_code": 'TRA',
        "transaction_type_code": 'INC',
        "uuid": "06f7eb1c-f1c5-42a9-94df-eaf384a91753"
    },
    "7064b536-82f9-498d-a5f7-cd2289680c95": {
        "amount": 21525.167,
        "description": "Realize blood alone enter.",
        "record_hash":
            "cac7baed55f36665f0c235a3471f6d7644b8929c1e70a72d4d95c3dea7196089",
        "reference_number": "KEQB8464688274218",
        "timestamp": 1564117807,
        "transaction_subcategory_code": 'MAG',
        "transaction_type_code": 'EXP',
        "uuid": "7064b536-82f9-498d-a5f7-cd2289680c95"
    },
    "7e6b739f-9c3a-434b-84dc-179fb841f8b0": {
        "amount": 594.405476893894,
        "description": "Simple join kid capital guess.",
        "record_hash":
            "bbe1c5d8f1a8e834bf84775a3b00b9d81034eb31656b69cad7099b391cbdc3d1",
        "reference_number": "YTAH8256952650808",
        "timestamp": 1562118663,
        "transaction_subcategory_code": 'SHO',
        "transaction_type_code": 'INC',
        "uuid": "7e6b739f-9c3a-434b-84dc-179fb841f8b0"
    },
    "135353d8-a97c-46d8-80a6-e272483c2913": {
        "amount": 29761.7948797,
        "description": "Close into short box.",
        "record_hash":
            "f17a6b034c2c0a24d9bfcbe4cc4532ed1cea93dd42ea7f1b15aa2395f5079ada",
        "reference_number": "SUHB3671579759896",
        "timestamp": 1562203602,
        "transaction_subcategory_code": 'TRA',
        "transaction_type_code": 'EXP',
        "uuid": "135353d8-a97c-46d8-80a6-e272483c2913"
    },
    "566d7fd9-7f22-4edf-be25-350e4a20e020": {
        "amount": 22577.87,
        "description": "About present night however either.",
        "record_hash":
            "58ee6f2d376ab09d29dd119bc0ba534d35e1a9f9f526a07bf80cfeae417322aa",
        "reference_number": "EZYE9034389895665",
        "timestamp": 1563475833,
        "transaction_subcategory_code": 'MUS',
        "transaction_type_code": 'EXP',
        "uuid": "566d7fd9-7f22-4edf-be25-350e4a20e020"
    },
    "c5e6fb07-4621-49e8-bd4f-0b8b6c2815fe": {
        "amount": 74025.946932,
        "description": "Hold newspaper tend operation myself.",
        "record_hash":
            "3d7255aa7d9d90bfa0929accf9423813393e47a094886b3e2f962d27b67794ad",
        "reference_number": "JUTS1147901870833",
        "timestamp": 1567043216,
        "transaction_subcategory_code": 'SHO',
        "transaction_type_code": 'INC',
        "uuid": "c5e6fb07-4621-49e8-bd4f-0b8b6c2815fe"
    },
    "92ac897b-c797-441b-bb83-4ef25aec3a28": {
        "amount": 18098.75404,
        "description": "Heavy clearly huge official.",
        "record_hash":
            "2756e6e2aa1140cc78a19ba58c9938caf3c21b4d14b9a42bfb8886e63d7bb474",
        "reference_number": "JIWY6072154633345",
        "timestamp": 1563386859,
        "transaction_subcategory_code": 'FUR',
        "transaction_type_code": 'EXP',
        "uuid": "92ac897b-c797-441b-bb83-4ef25aec3a28"
    },
    "e9e5d51a-2ed4-4cea-941e-01387c29ed63": {
        "amount": 48066.6168102,
        "description": "Science pass make poor.",
        "record_hash":
            "778a9836ceac2c9f118d915d54f303b5538750e0c26e7b1d0da5cadf38dc2478",
        "reference_number": "OSZZ2507659238321",
        "timestamp": 1563691511,
        "transaction_subcategory_code": 'FUR',
        "transaction_type_code": 'INC',
        "uuid": "e9e5d51a-2ed4-4cea-941e-01387c29ed63"
    },
    "a59359cb-e3dd-40ba-99c6-023b1f71dba8": {
        "amount": 95916.2604,
        "description": "Become air until.",
        "record_hash":
            "02232713a7fc0a29ddb6c3295b8307b31f989f76e8d51a2620fe541c05cd50f8",
        "reference_number": "BSTX3371171444377",
        "timestamp": 1565005560,
        "transaction_subcategory_code": 'FUR',
        "transaction_type_code": 'EXP',
        "uuid": "a59359cb-e3dd-40ba-99c6-023b1f71dba8"
    },
    "09bff241-88c7-46be-bd3f-cf99311216f1": {
        "amount": 72430,
        "description": "List buy line three.",
        "record_hash":
            "b431d9620ae1c48ca7999b9fdd0ed3eb718d1272945a6e29e0c7ad15bb01d3ca",
        "reference_number": "VTZU6120647604067",
        "timestamp": 1564125154,
        "transaction_subcategory_code": 'FOD',
        "transaction_type_code": 'EXP',
        "uuid": "09bff241-88c7-46be-bd3f-cf99311216f1"
    },
    "a00a90f0-861c-4bda-8d1a-b6a78a61472f": {
        "amount": 37596.13,
        "description": "Sure amount buy instead smile play something.",
        "record_hash":
            "f5b6544e4f730cb90a6be60fc8b89abb82643d0064e3bebcefe81e23f0e9b5e5",
        "reference_number": "HNDE2061833973130",
        "timestamp": 1562284203,
        "transaction_subcategory_code": 'MOV',
        "transaction_type_code": 'INC',
        "uuid": "a00a90f0-861c-4bda-8d1a-b6a78a61472f"
    },
    "0afe80ca-245a-4d38-b610-b79fc96c9e81": {
        "amount": 71142.365,
        "description":
            "Professor pass event short particular consider several.",
        "record_hash":
            "2c695ddc76fbb44b16e51055da2d2914e860281ba822393cee5466060e1508d4",
        "reference_number": "XIAF8021720287232",
        "timestamp": 1563050948,
        "transaction_subcategory_code": 'TRA',
        "transaction_type_code": 'EXP',
        "uuid": "0afe80ca-245a-4d38-b610-b79fc96c9e81"
    },
    "f2a25f46-6f93-46c9-bca2-4f613c6f77cc": {
        "amount": 81303.159,
        "description": "Well friend because support economic Republican.",
        "record_hash":
            "43803c7076fcab9548003c0eb9bba7a74fa5d77e5d6afc32843cb36ef37a0acd",
        "reference_number": "GZWP2774492450280",
        "timestamp": 1567132581,
        "transaction_subcategory_code": 'RES',
        "transaction_type_code": 'EXP',
        "uuid": "f2a25f46-6f93-46c9-bca2-4f613c6f77cc"
    },
    "805458cf-fb60-4bfd-b013-c12c294ce8e2": {
        "amount": 12188.1,
        "description": "Ever animal morning.",
        "record_hash":
            "a18be4d819cea919c341aafc85ddfd9071d9ccd5ee69f1fd1278d523fa85bd83",
        "reference_number": "MGMY0829874488700",
        "timestamp": 1563781154,
        "transaction_subcategory_code": 'GIF',
        "transaction_type_code": 'INC',
        "uuid": "805458cf-fb60-4bfd-b013-c12c294ce8e2"
    },
    "605de309-c799-4350-9b9b-35a69aa7f4d7": {
        "amount": 48859.78,
        "description": "Value cover analysis smile.",
        "record_hash":
            "7ee740d50a3194c6aea0f39dba5bb64d310d79e4a439d56c5798014d4f88a77e",
        "reference_number": "WXMH3690572168023",
        "timestamp": 1564522665,
        "transaction_subcategory_code": 'REP',
        "transaction_type_code": 'INC',
        "uuid": "605de309-c799-4350-9b9b-35a69aa7f4d7"
    },
    "3455056a-8d81-4341-95a5-6347f6bc6653": {
        "amount": 50445.461,
        "description": "Future if you its put team Congress last.",
        "record_hash":
            "735450f55e8729be18ea8ce131fac1e6155cc42e90536147fe3cbebe81f30f11",
        "reference_number": "QYAB8226384924840",
        "timestamp": 1564210602,
        "transaction_subcategory_code": 'GIF',
        "transaction_type_code": 'INC',
        "uuid": "3455056a-8d81-4341-95a5-6347f6bc6653"
    }
}



from kakeibox_core.use_cases.expense.update import UpdateExpense
from kakeibox_storage_memory.bridge import Bridge
from kakeibox_storage_memory.storage import StorageTransaction
from faker import Faker
import datetime


def before_execute_test():
    storage = StorageTransaction()
    storage.table = {}


def test_use_case_expense_update(an_expense_data):
    before_execute_test()
    fake = Faker()
    storage = StorageTransaction()
    storage.table[an_expense_data['uuid']] = an_expense_data
    assert 1 == len(storage.table)

    end = datetime.datetime.now()
    start = end - datetime.timedelta(days=10)

    an_expense_data['description'] = fake.sentence()
    an_expense_data['amount'] = fake.pydecimal(
        positive=True, min_value=0, max_value=10000)
    an_expense_data['timestamp'] = fake.unix_time(
        start_datetime=start, end_datetime=end)
    an_expense_data['reference_number'] = fake.bban()

    storage_bridge = Bridge()
    use_case = UpdateExpense(storage_bridge)
    result = use_case.execute(an_expense_data['uuid'], an_expense_data)
    expected = storage.table[an_expense_data['uuid']]

    assert result
    assert isinstance(result.value, dict)
    assert expected['uuid'] == result.value['uuid']
    assert expected['description'] == result.value['description']
    assert expected['amount'] == result.value['amount']
    assert expected['timestamp'] == result.value['timestamp']
    assert expected['reference_number'] == result.value['reference_number']
    assert expected['record_hash'] == result.value['record_hash']
    assert expected['transaction_type_code'] == result.value[
        'transaction_type_code']
    assert expected['transaction_subcategory_code'] == result.value[
        'transaction_subcategory_code']



from kakeibox_core.use_cases.expense.new import NewExpense
from kakeibox_storage_memory.bridge import Bridge
from kakeibox_storage_memory.storage import StorageTransaction


def before_execute_test():
    storage = StorageTransaction()
    storage.table = {}


def test_use_case_expense_new(an_expense_data):
    before_execute_test()
    transaction_dict = an_expense_data
    storage = StorageTransaction()
    assert 0 == len(storage.table)

    storage_bridge = Bridge()
    use_case = NewExpense(storage_bridge)
    result = use_case.execute(transaction_dict)
    assert result
    assert isinstance(result.value, dict)

    storage = StorageTransaction()
    assert 1 == len(storage.table)

    new_one = storage.table[transaction_dict['uuid']]
    assert result.value['uuid'] == new_one['uuid']
    assert result.value['description'] == new_one['description']
    assert result.value['amount'] == new_one['amount']
    assert result.value['timestamp'] == new_one['timestamp']
    assert result.value['record_hash'] == new_one['record_hash']
    assert result.value['reference_number'] == new_one['reference_number']
    assert result.value['transaction_type_code'] == new_one[
        'transaction_type_code']
    assert result.value['transaction_subcategory_code'] == new_one[
        'transaction_subcategory_code']



from kakeibox_core.use_cases.expense.total import GetTotalExpense
from kakeibox_storage_memory.bridge import Bridge
from kakeibox_storage_memory.storage import StorageTransaction
import datetime


def before_execute_test():
    storage = StorageTransaction()
    storage.table = {}


def test_use_case_expense_total(transaction_total_data):
    before_execute_test()
    end_time = datetime.datetime.now()
    start_time = (end_time - datetime.timedelta(days=10)).timestamp()
    end_time = end_time.timestamp()

    storage = StorageTransaction()
    storage.table = transaction_total_data
    expenses = [item for item in storage.table.values()
                if int(start_time) <= int(item['timestamp']) <= int(end_time)
                and item['transaction_type_code'] == 'EXP'
                ]
    total_expense = 0
    for expense in expenses:
        total_expense += expense['amount']
    total_expense = round(total_expense, 2)

    storage_bridge = Bridge()
    use_case = GetTotalExpense(storage_bridge)
    result = use_case.execute(start_time, end_time)
    assert result
    assert isinstance(result.value, float)
    assert total_expense == result.value



from kakeibox_core.use_cases.expense.delete import DeleteExpense
from kakeibox_storage_memory.bridge import Bridge
from kakeibox_storage_memory.storage import StorageTransaction


def before_execute_test():
    storage = StorageTransaction()
    storage.table = {}


def test_use_case_expense_delete(an_expense_data):
    before_execute_test()
    item = an_expense_data
    storage = StorageTransaction()
    storage.table[item['uuid']] = item
    assert 1 == len(storage.table)

    storage_bridge = Bridge()
    use_case = DeleteExpense(storage_bridge)
    result = use_case.execute(item['uuid'])
    assert result
    assert isinstance(result.value, bool)
    assert result.value
    assert 0 == len(storage.table)



from dependency_injector import containers, providers
from kakeibox_core import use_cases
from kakeibox_core.entry_points.adapters import Adapters, TestAdapters


class Commands(containers.DeclarativeContainer):

    adapters = providers.DependenciesContainer()

    # transaction_category use cases
    new_category = providers.Factory(
        use_cases.transaction_category.new.NewTransactionCategory,
        storage_bridge=adapters.storage_bridge)

    update_category = providers.Factory(
        use_cases.transaction_category.update.UpdateTransactionCategory,
        storage_bridge=adapters.storage_bridge)

    get_category = providers.Factory(
        use_cases.transaction_category.get.TransactionCategoryByCode,
        storage_bridge=adapters.storage_bridge
    )

    list_categories = providers.Factory(
        use_cases.transaction_category.list.ListTransactionCategories,
        storage_bridge=adapters.storage_bridge
    )

    delete_category = providers.Factory(
        use_cases.transaction_category.delete.DeleteTransactionCategory,
        storage_bridge=adapters.storage_bridge)

    # transaction_subcategory use cases
    new_subcategory = providers.Factory(
        use_cases.transaction_subcategory.new.NewTransactionSubcategory,
        storage_bridge=adapters.storage_bridge)

    update_subcategory = providers.Factory(
        use_cases.transaction_subcategory.update.UpdateTransactionSubcategory,
        storage_bridge=adapters.storage_bridge)

    get_subcategory = providers.Factory(
        use_cases.transaction_subcategory.get.TransactionSubcategoryByCode,
        storage_bridge=adapters.storage_bridge
    )

    list_all_subcategories = providers.Factory(
        use_cases.transaction_subcategory.list_all.
            ListAllTransactionSubcategories,
        storage_bridge=adapters.storage_bridge
    )

    list_per_category = providers.Factory(
        use_cases.transaction_subcategory.list_per_category.
            ListTransactionSubcategoriesPerCategory,
        storage_bridge=adapters.storage_bridge
    )

    delete_subcategory = providers.Factory(
        use_cases.transaction_subcategory.delete.DeleteTransactionSubcategory,
        storage_bridge=adapters.storage_bridge)

    # transaction use cases
    list_transactions = providers.Factory(
        use_cases.transaction.list.ListTransactions,
        storage_bridge=adapters.storage_bridge)

    # saving use cases
    calculate_savings = providers.Factory(
        use_cases.saving.total.CalculateSaving,
        storage_bridge=adapters.storage_bridge)

    # income use cases
    new_income = providers.Factory(
        use_cases.income.new.NewIncome,
        storage_bridge=adapters.storage_bridge)

    update_income = providers.Factory(
        use_cases.income.update.UpdateIncome,
        storage_bridge=adapters.storage_bridge)

    get_income = providers.Factory(
        use_cases.income.get.GetIncomeByUUID,
        storage_bridge=adapters.storage_bridge
    )

    calculate_total_income = providers.Factory(
        use_cases.income.total.GetTotalIncome,
        storage_bridge=adapters.storage_bridge
    )

    delete_income = providers.Factory(
        use_cases.income.delete.DeleteIncome,
        storage_bridge=adapters.storage_bridge)

    # expense use cases
    new_expense = providers.Factory(
        use_cases.expense.new.NewExpense,
        storage_bridge=adapters.storage_bridge)

    update_expense = providers.Factory(
        use_cases.expense.update.UpdateExpense,
        storage_bridge=adapters.storage_bridge)

    get_expense = providers.Factory(
        use_cases.expense.get.GetExpenseByUUID,
        storage_bridge=adapters.storage_bridge
    )

    calculate_total_expense = providers.Factory(
        use_cases.expense.total.GetTotalExpense,
        storage_bridge=adapters.storage_bridge
    )

    delete_expense = providers.Factory(
        use_cases.expense.delete.DeleteExpense,
        storage_bridge=adapters.storage_bridge)


commands = Commands(adapters=Adapters())



from kakeibox_core.use_cases.transaction_subcategory.update import \
    UpdateTransactionSubcategory
from kakeibox_storage_memory.storage import StorageTransactionSubcategory
from kakeibox_storage_memory.bridge import Bridge
from time import time


def before_execute_test():
    storage = StorageTransactionSubcategory()
    storage.table = {}


def test_use_case_transaction_subcategory_update(transaction_subcategory_data):
    before_execute_test()
    item = list(transaction_subcategory_data.values())[0]
    storage = StorageTransactionSubcategory()
    storage.table[item['code']] = item
    assert 1 == len(storage.table)

    storage_bridge = Bridge()
    item['name'] = "new name {}".format(time())
    use_case = UpdateTransactionSubcategory(storage_bridge)
    result = use_case.execute(item['code'], item)
    expected = storage.table[item['code']]

    assert result
    assert isinstance(result.value, dict)
    assert expected['code'] == result.value['code']
    assert expected['name'] == result.value['name']
